@@ -498,6 +498,35 @@ def _AddInitMethod(message_descriptor, cls):
     return value
 
   def init(self, **kwargs):
+
+    def init_wkt_or_merge(field, msg, value):
+      if isinstance(value, message_mod.Message):
+        msg.MergeFrom(value)
+      elif (
+          isinstance(value, dict)
+          and field.message_type.full_name == _StructFullTypeName
+      ):
+        msg.Clear()
+        if len(value) == 1 and 'fields' in value:
+          try:
+            msg.update(value)
+          except:
+            msg.Clear()
+            msg.__init__(**value)
+        else:
+          msg.update(value)
+      elif hasattr(msg, '_internal_assign'):
+        msg._internal_assign(value)
+      else:
+        raise TypeError(
+            'Message field {0}.{1} must be initialized with a '
+            'dict or instance of same class, got {2}.'.format(
+                message_descriptor.name,
+                field.name,
+                type(value).__name__,
+            )
+        )
+
     self._cached_byte_size = 0
     self._cached_byte_size_dirty = len(kwargs) > 0
     self._fields = {}
@@ -534,10 +563,13 @@ def _AddInitMethod(message_descriptor, cls):
               field_copy.update(field_value)
           else:
             for val in field_value:
-              if isinstance(val, dict):
+              if isinstance(val, dict) and (
+                  field.message_type.full_name != _StructFullTypeName
+              ):
                 field_copy.add(**val)
               else:
-                field_copy.add().MergeFrom(val)
+                new_msg = field_copy.add()
+                init_wkt_or_merge(field, new_msg, val)
         else:  # Scalar
           if field.cpp_type == _FieldDescriptor.CPPTYPE_ENUM:
             field_value = [_GetIntegerEnumValue(field.enum_type, val)
@@ -546,38 +578,14 @@ def _AddInitMethod(message_descriptor, cls):
         self._fields[field] = field_copy
       elif field.cpp_type == _FieldDescriptor.CPPTYPE_MESSAGE:
         field_copy = field._default_constructor(self)
-        new_val = None
-        if isinstance(field_value, message_mod.Message):
-          new_val = field_value
-        elif isinstance(field_value, dict):
-          if field.message_type.full_name == _StructFullTypeName:
-            field_copy.Clear()
-            if len(field_value) == 1 and 'fields' in field_value:
-              try:
-                field_copy.update(field_value)
-              except:
-                # Fall back to init normal message field
-                field_copy.Clear()
-                new_val = field.message_type._concrete_class(**field_value)
-            else:
-              field_copy.update(field_value)
-          else:
-            new_val = field.message_type._concrete_class(**field_value)
-        elif hasattr(field_copy, '_internal_assign'):
-          field_copy._internal_assign(field_value)
+        if isinstance(field_value, dict) and (
+            field.message_type.full_name != _StructFullTypeName
+        ):
+          new_val = field.message_type._concrete_class(**field_value)
+          field_copy.MergeFrom(new_val)
         else:
-          raise TypeError(
-              'Message field {0}.{1} must be initialized with a '
-              'dict or instance of same class, got {2}.'.format(
-                  message_descriptor.name,
-                  field_name,
-                  type(field_value).__name__,
-              )
-          )
-
-        if new_val != None:
           try:
-            field_copy.MergeFrom(new_val)
+            init_wkt_or_merge(field, field_copy, field_value)
           except TypeError:
             _ReraiseTypeErrorWithFieldName(message_descriptor.name, field_name)
         self._fields[field] = field_copy
